@@ -167,6 +167,7 @@ int ksmbd_verify_smb_message(struct ksmbd_work *work)
 		return ksmbd_smb2_check_message(work);
 	}
 
+	work->conn->outstanding_credits++;
 	return ksmbd_smb1_check_message(work);
 #else
 	struct smb_hdr *hdr;
@@ -176,8 +177,10 @@ int ksmbd_verify_smb_message(struct ksmbd_work *work)
 
 	hdr = work->request_buf;
 	if (*(__le32 *)hdr->Protocol == SMB1_PROTO_NUMBER &&
-	    hdr->Command == SMB_COM_NEGOTIATE)
+	    hdr->Command == SMB_COM_NEGOTIATE) {
+		work->conn->outstanding_credits++;
 		return 0;
+	}
 
 	return -EINVAL;
 #endif
@@ -333,7 +336,7 @@ int ksmbd_init_smb_server(struct ksmbd_work *work)
 	if (proto == SMB1_PROTO_NUMBER)
 		init_smb1_server(conn);
 	else
-		init_smb2_0_server(conn);
+		init_smb3_11_server(conn);
 #else
 	init_smb3_11_server(conn);
 #endif
@@ -363,14 +366,17 @@ int ksmbd_populate_dot_dotdot_entries(struct ksmbd_work *work, int info_level,
 	for (i = 0; i < 2; i++) {
 		struct kstat kstat;
 		struct ksmbd_kstat ksmbd_kstat;
+		struct dentry *dentry;
 
 		if (!dir->dot_dotdot[i]) { /* fill dot entry info */
 			if (i == 0) {
 				d_info->name = ".";
 				d_info->name_len = 1;
+				dentry = dir->filp->f_path.dentry;
 			} else {
 				d_info->name = "..";
 				d_info->name_len = 2;
+				dentry = dir->filp->f_path.dentry->d_parent;
 			}
 
 			if (!match_pattern(d_info->name, d_info->name_len,
@@ -382,7 +388,7 @@ int ksmbd_populate_dot_dotdot_entries(struct ksmbd_work *work, int info_level,
 			ksmbd_kstat.kstat = &kstat;
 			ksmbd_vfs_fill_dentry_attrs(work,
 						    user_ns,
-						    dir->filp->f_path.dentry->d_parent,
+						    dentry,
 						    &ksmbd_kstat);
 			rc = fn(conn, info_level, d_info, &ksmbd_kstat);
 			if (rc)
