@@ -6,9 +6,13 @@
  *   Author(s): Namjae Jeon <linkinjeon@kernel.org>
  */
 
+#include <linux/version.h>
 #include <linux/fs.h>
 #include <linux/slab.h>
 #include <linux/string.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 17, 0)
+#include <linux/mnt_idmapping.h>
+#endif
 
 #include "smbacl.h"
 #include "smb_common.h"
@@ -277,6 +281,9 @@ static int sid_to_id(struct user_namespace *user_ns,
 		uid_t id;
 
 		id = le32_to_cpu(psid->sub_auth[psid->num_subauth - 1]);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 17, 0)
+		uid = mapped_kuid_user(user_ns, &init_user_ns, KUIDT_INIT(id));
+#else
 		/*
 		 * Translate raw sid into kuid in the server's user
 		 * namespace.
@@ -286,6 +293,7 @@ static int sid_to_id(struct user_namespace *user_ns,
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 12, 0)
 		/* If this is an idmapped mount, apply the idmapping. */
 		uid = kuid_from_mnt(user_ns, uid);
+#endif
 #endif
 		if (uid_valid(uid)) {
 			fattr->cf_uid = uid;
@@ -296,6 +304,9 @@ static int sid_to_id(struct user_namespace *user_ns,
 		gid_t id;
 
 		id = le32_to_cpu(psid->sub_auth[psid->num_subauth - 1]);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 17, 0)
+		gid = mapped_kgid_user(user_ns, &init_user_ns, KGIDT_INIT(id));
+#else
 		/*
 		 * Translate raw sid into kgid in the server's user
 		 * namespace.
@@ -305,6 +316,7 @@ static int sid_to_id(struct user_namespace *user_ns,
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 12, 0)
 		/* If this is an idmapped mount, apply the idmapping. */
 		gid = kgid_from_mnt(user_ns, gid);
+#endif
 #endif
 		if (gid_valid(gid)) {
 			fattr->cf_gid = gid;
@@ -1281,6 +1293,7 @@ int smb_check_perm_dacl(struct ksmbd_conn *conn, struct path *path,
 					if (!access_bits)
 						access_bits =
 							SET_MINIMUM_RIGHTS;
+					posix_acl_release(posix_acls);
 					goto check_access_bits;
 				}
 			}
