@@ -12,6 +12,9 @@
 #include <linux/fs.h>
 #include <linux/namei.h>
 #include <linux/posix_acl.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 17, 0)
+#include <linux/mnt_idmapping.h>
+#endif
 
 #include "mgmt/tree_connect.h"
 
@@ -218,7 +221,11 @@ static inline uid_t posix_acl_uid_translate(struct user_namespace *mnt_userns,
 	kuid_t kuid;
 
 	/* If this is an idmapped mount, apply the idmapping. */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 17, 0)
+	kuid = mapped_kuid_fs(mnt_userns, &init_user_ns, pace->e_uid);
+#else
 	kuid = kuid_into_mnt(mnt_userns, pace->e_uid);
+#endif
 
 	/* Translate the kuid into a userspace id ksmbd would see. */
 	return from_kuid(&init_user_ns, kuid);
@@ -234,7 +241,11 @@ static inline gid_t posix_acl_gid_translate(struct user_namespace *mnt_userns,
 	kgid_t kgid;
 
 	/* If this is an idmapped mount, apply the idmapping. */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 17, 0)
+	kgid = mapped_kgid_fs(mnt_userns, &init_user_ns, pace->e_gid);
+#else
 	kgid = kgid_into_mnt(mnt_userns, pace->e_gid);
+#endif
 
 	/* Translate the kgid into a userspace id ksmbd would see. */
 	return from_kgid(&init_user_ns, kgid);
